/*
 * Copyright 2006 Takahiro Nakamura.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package woolpack.utils;

import java.io.IOException;
import java.io.InputStream;

/**
 * {@link InputStream}のファクトリ。
 * @author nakamura
 *
 */
public interface InputStreamFactory {
	/**
	 * {@link InputStream}を生成して返す。
	 * 呼び出し元は本メソッドの返却値が null である可能性を考慮する必要がある。
	 * @param id キー。
	 * @return {@link InputStream}。
	 * @throws IOException {@link InputStream}の生成に失敗した場合。
	 */
	public InputStream newInstance(String id) throws IOException;
}
